Package: modsem
Type: Package
Title: Latent Interaction (and Moderation) Analysis in Structural
        Equation Models (SEM)
Version: 1.0.22
Authors@R: 
    c(person(given = "Kjell", family = "Solem Slupphaug", 
             email = "slupphaugkjell@gmail.com", role = c("aut", "cre"),
             comment = c(ORCID = "0009-0005-8324-2834")),
      person(given = "Mehmet", family="Mehmetoglu",
             email = "mehmetm@ntnu.no", role=c("ctb"),
             comment = c(ORCID="0000-0002-6092-8551")),
      person(given = "Matthias", family="Mittner",
             email = "matthias.mittner@uit.no", role=c("ctb"),
             comment = c(ORCID="0000-0003-0205-7353")))
Maintainer: Kjell Solem Slupphaug <slupphaugkjell@gmail.com>
Description: 
    Estimation of interaction (i.e., moderation) effects between latent variables
    in structural equation models (SEM). 
    The supported methods are:
      The constrained approach (Algina & Moulder, 2001).
      The unconstrained approach (Marsh et al., 2004).
      The residual centering approach (Little et al., 2006).
      The double centering approach (Lin et al., 2010).
      The latent moderated structural equations (LMS) approach (Klein & Moosbrugger, 2000).
      The quasi-maximum likelihood (QML) approach (Klein & Muthén, 2007)
    The constrained- unconstrained, residual- and double centering- approaches
    are estimated via 'lavaan' (Rosseel, 2012), whilst the LMS- and QML- approaches
    are estimated via 'modsem' it self. Alternatively model can be
    estimated via 'Mplus' (Muthén & Muthén, 1998-2017).
    References:
    Algina, J., & Moulder, B. C. (2001). 
      <doi:10.1207/S15328007SEM0801_3>.
      "A note on estimating the Jöreskog-Yang model for latent variable interaction using 'LISREL' 8.3."
    Klein, A., & Moosbrugger, H. (2000). 
      <doi:10.1007/BF02296338>.
      "Maximum likelihood estimation of latent interaction effects with the LMS method."
    Klein, A. G., & Muthén, B. O. (2007). 
      <doi:10.1080/00273170701710205>.
      "Quasi-maximum likelihood estimation of structural equation models with multiple interaction and quadratic effects."
    Lin, G. C., Wen, Z., Marsh, H. W., & Lin, H. S. (2010). 
      <doi:10.1080/10705511.2010.488999>.
      "Structural equation models of latent interactions: Clarification of orthogonalizing and double-mean-centering strategies."
    Little, T. D., Bovaird, J. A., & Widaman, K. F. (2006). 
      <doi:10.1207/s15328007sem1304_1>.
      "On the merits of orthogonalizing powered and product terms: Implications for modeling interactions among latent variables."
    Marsh, H. W., Wen, Z., & Hau, K. T. (2004). 
      <doi:10.1037/1082-989X.9.3.275>.
      "Structural equation models of latent interactions: evaluation of alternative estimation strategies and indicator construction."
    Muthén, L.K. and Muthén, B.O. (1998-2017).  
      "'Mplus' User’s Guide. Eighth Edition."
      <https://www.statmodel.com/>.
    Rosseel Y (2012). 
      <doi:10.18637/jss.v048.i02>.
      "'lavaan': An R Package for Structural Equation Modeling." 
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, purrr, stringr, lavaan, rlang, nlme, dplyr, mvnfast,
        stats, fastGHQuad, mvtnorm, ggplot2, parallel, plotly, Deriv,
        MASS, Amelia, grDevices, cli, RhpcBLASctl, memoise
Depends: R (>= 4.1.0)
URL: https://modsem.org, https://github.com/kss2k/modsem
Suggests: knitr, rmarkdown, ggpubr, RColorBrewer, MplusAutomation
VignetteBuilder: knitr
Config/roxygen2/version: 8.1.0
NeedsCompilation: yes
Packaged: 2026-08-20 12:57:39 UTC; kss
Author: Kjell Solem Slupphaug [aut, cre] (ORCID:
    <https://orcid.org/0009-0005-8324-2834>),
  Mehmet Mehmetoglu [ctb] (ORCID:
    <https://orcid.org/0000-0002-6092-8551>),
  Matthias Mittner [ctb] (ORCID: <https://orcid.org/0000-0003-0205-7353>)
Repository: CRAN
Date/Publication: 2026-08-21 07:00:09 UTC
