#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @name modsem
#' @importFrom Rcpp sourceCpp
#' @importFrom memoise memoise
#' @useDynLib modsem, .registration = TRUE
## usethis namespace: end
NULL
