---
title: "using lavaan functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{using lavaan functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
EVAL_DEFAULT <- FALSE
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = EVAL_DEFAULT
)
```

```{r setup}
library(modsem)
```
If you're using one of the product indicator approaches, you might want to use some lavaan
functions on top of the estimated lavaan-object.
To do so you can extract the lavaan-object using the `extract_lavaan()` function.

```{r}
library(lavaan)

m1 <- '
# Outer Model
X =~ x1 + x2 + x3
Y =~ y1 + y2 + y3
Z =~ z1 + z2 + z3

# Inner model
Y ~ X + Z + X:Z
'

est <- modsem(m1, oneInt)
lav_object <- extract_lavaan(est)
bootstrap <- bootstrapLavaan(lav_object, R = 500)
```
