% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{jordan}
\alias{jordan}
\title{Jordan subset of PISA 2006 data}
\format{
A data frame of fifteen variables and 6,038 observations:

\describe{
\item{enjoy1}{indicator for enjoyment of science, item ST16Q01: I generally have fun when I am learning <broad science> topics.}
\item{enjoy2}{indicator for enjoyment of science, item ST16Q02: I like reading about <broad science>.}
\item{enjoy3}{indicator for enjoyment of science, item ST16Q03: I am happy doing <broad science> problems.}
\item{enjoy4}{indicator for enjoyment of science, item ST16Q04: I enjoy acquiring new knowledge in <broad science>.}
\item{enjoy5}{indicator for enjoyment of science, item ST16Q05: I am interested in learning about <broad science>.}
\item{academic1}{indicator for academic self-concept in science, item ST37Q01: I can easily understand new ideas in <school science>.}
\item{academic2}{indicator for academic self-concept in science, item ST37Q02: Learning advanced <school science> topics would be easy for me.}
\item{academic3}{indicator for academic self-concept in science, item ST37Q03: I can usually give good answers to <test questions> on <school science> topics.}
\item{academic4}{indicator for academic self-concept in science, item ST37Q04: I learn <school science> topics quickly.}
\item{academic5}{indicator for academic self-concept in science, item ST37Q05: <School science> topics are easy for me.}
\item{academic6}{indicator for academic self-concept in science, item ST37Q06: When I am being taught <school science>, I can understand the concepts very well.}
\item{career1}{indicator for career aspirations in science, item ST29Q01: I would like to work in a career involving <broad science>.}
\item{career2}{indicator for career aspirations in science, item ST29Q02: I would like to study <broad science> after <secondary school>.}
\item{career3}{indicator for career aspirations in science, item ST29Q03: I would like to spend my life doing advanced <broad science>.}
\item{career4}{indicator for career aspirations in science, item ST29Q04: I would like to work on <broad science> projects as an adult.}
}
}
\source{
This version of the dataset, as well as the description was gathered from the
documentation of the 'nlsem' package (https://cran.r-project.org/package=nlsem),
where the only difference is that the names of the variables were changed

Originally the dataset was gathered by the Organisation for Economic Co-Operation and Development (2009).
Pisa 2006: Science competencies for tomorrow's world (Tech. Rep.).
Paris, France. Obtained from: https://www.oecd.org/pisa/pisaproducts/database-pisa2006.htm
}
\description{
The data stem from the large-scale assessment study PISA 2006
(Organisation for Economic Co-Operation and Development, 2009) where
competencies of 15-year-old students in reading, mathematics, and science
are assessed using nationally representative samples in 3-year cycles.
In this example, data from the student background questionnaire from the
Jordan sample of PISA 2006 were used. Only data of students with complete
responses to all 15 items (N = 6,038) were considered.
}
\examples{
\dontrun{
m1 <- "
  ENJ =~ enjoy1 + enjoy2 + enjoy3 + enjoy4 + enjoy5
  CAREER =~ career1 + career2 + career3 + career4
  SC =~ academic1 + academic2 + academic3 + academic4 + academic5 + academic6
  CAREER ~ ENJ + SC + ENJ:ENJ + SC:SC + ENJ:SC
"

est <- modsem(m1, data = jordan, method = "qml")
summary(est)
}
}
